<?php

namespace App\Entity;

use App\Repository\FilmRepository;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

#[ORM\Table(name: "film")]
#[ORM\Entity(repositoryClass: FilmRepository::class)]
class Film
{
    #[ORM\Column(name: "id", type: "integer")]
    #[ORM\Id]
    #[ORM\GeneratedValue(strategy: "AUTO")]
    private $id;

    #[ORM\Column(name: "titre", type: "string", length: 191)]
    private $titre;

    #[ORM\Column(name: "annee", type: "date", nullable: true)]
    private $annee;

    #[ORM\Column(name: "date_submited", type: "datetime", nullable: true)]
    private $dateSubmited;

    #[ORM\Column(name: "lien", type: "string", length: 191, nullable: true)]
    #[Assert\Url()]
    private $lien;

    #[ORM\Column(name: "note", type: "float", nullable: true)]
    private $note;

    #[ORM\Column(name: "nb_coms", type: "integer", nullable: true)]
    private $nbComs;

    #[Assert\Valid()]
    #[ORM\OneToOne(targetEntity: MediaVideo::class, cascade: ["persist","remove"])]
    #[ORM\JoinColumn(nullable: true)]
    private $mediaVideo;

    #[ORM\ManyToMany(targetEntity: Realisateur::class, inversedBy: "films", cascade: ["persist"])]
    private $realisateurs;

    #[ORM\OneToMany(targetEntity: Commentaire::class, mappedBy: "film", orphanRemoval: true)]
    private $commentaires;

    #[ORM\ManyToOne(targetEntity: User::class)]
    #[ORM\JoinColumn(nullable: true)]
    private $authered;

    #[ORM\ManyToMany(targetEntity: User::class, inversedBy: "films")]
    #[ORM\JoinTable(name: "filmsavoir_users")]
    private $usersWantToView;

    #[ORM\ManyToMany(targetEntity: User::class, inversedBy: "filmsVus")]
    #[ORM\JoinTable(name: "filmsvus_users")]
    private $usersWhoSeen;

    #[ORM\ManyToMany(targetEntity: Genre::class, cascade: ["persist"])]
    private $genres;

    #[ORM\Column(type: "text", nullable: true)]
    private $information;

    #[ORM\Column(type: "date", nullable: true)]
    private $dateSortie;

    public function __construct()
    {
        $this->realisateurs = new \Doctrine\Common\Collections\ArrayCollection();
        $this->usersWantToView = new \Doctrine\Common\Collections\ArrayCollection();
        $this->usersWhoSeen = new \Doctrine\Common\Collections\ArrayCollection();
        $this->genres = new \Doctrine\Common\Collections\ArrayCollection();
        $this->commentaires = new \Doctrine\Common\Collections\ArrayCollection();
        $this->setDateSubmited(new \DateTime());
    }

    public function getAuthered()
    {
        return $this->authered;
    }

    public function setAuthered(User $authered)
    {
        $this->authered = $authered;
    }

    public function getId()
    {
        return $this->id;
    }

    public function setTitre($titre)
    {
        $this->titre = $titre;

        return $this;
    }

    public function getTitre()
    {
        return $this->titre;
    }

    public function setAnnee($annee)
    {
        $this->annee = $annee;

        return $this;
    }

    public function getAnnee()
    {
        return $this->annee;
    }

    public function getLien()
    {
        return $this->lien;
    }

    public function setLien($lien = null)
    {
        $this->lien = $lien;
    }

    public function getDateSubmited()
    {
        return $this->dateSubmited;
    }

    public function setDateSubmited(\DateTime $dateSubmited)
    {
        $this->dateSubmited = $dateSubmited;
    }

    public function isNew()
    {
        $finNew = $this->getDateSubmited()->modify('+1 day');
        return (new \DateTime('now') < $finNew);
    }

    public function addCommentaire(\App\Entity\Commentaire $commentaire)
    {
        $this->commentaires[] = $commentaire;

        return $this;
    }

    public function removeCommentaire(\App\Entity\Commentaire $commentaire)
    {
        $this->commentaires->removeElement($commentaire);

        return $this;
    }

    public function getCommentaires()
    {
        return $this->commentaires;
    }

    public function addRealisateur(Realisateur $realisateur)
    {
        $this->realisateurs[] = $realisateur;

        return $this;
    }

    public function removeRealisateur(Realisateur $realisateur)
    {
        $this->realisateurs->removeElement($realisateur);
    }

    public function getRealisateurs()
    {
        return $this->realisateurs;
    }

    public function addUserWantToView(User $user)
    {
        $this->usersWantToView[] = $user;
        $user->addFilm($this);

        return $this;
    }

    public function removeUserWantToView(User $user)
    {
        $this->usersWantToView->removeElement($user);
        $user->removeFilm($this);
    }

    public function getUsersWantToView()
    {
        return $this->usersWantToView;
    }

    public function addUserWhoSeen(User $user)
    {
        $this->usersWhoSeen[] = $user;
        $user->addFilmVu($this);

        return $this;
    }

    public function removeUserWhoSeen(User $user)
    {
        $this->usersWhoSeen->removeElement($user);
        $user->removeFilmVu($this);
    }

    public function getUsersWhoSeen()
    {
        return $this->usersWhoSeen;
    }

    public function getGenres()
    {
        return $this->genres;
    }

    public function addGenre(Genre $genre)
    {
        $this->genres[] = $genre;
        return $this;
    }

    public function removeGenre(Genre $genre)
    {
        $this->genres->removeElement($genre);
        return $this;
    }

    public function getNote()
    {
        return $this->note;
    }

    public function setNote($note)
    {
        $this->note = $note;
    }

    public function getNbComs()
    {
        return $this->nbComs;
    }

    public function setNbComs($nbComs)
    {
        $this->nbComs = $nbComs;
    }

    public function getInformation(): ?string
    {
        return $this->information;
    }

    public function setInformation(?string $information): self
    {
        $this->information = $information;

        return $this;
    }

    public function getDateSortie(): ?\DateTimeInterface
    {
        return $this->dateSortie;
    }

    public function setDateSortie(?\DateTimeInterface $dateSortie): self
    {
        $this->dateSortie = $dateSortie;

        return $this;
    }

    public function getMediaVideo()
    {
        return $this->mediaVideo;
    }

    public function setMediaVideo(MediaVideo $mediaVideo = null)
    {
        $this->mediaVideo = $mediaVideo;
    }


}
