<?php

namespace App\Entity;

use App\Repository\UserRepository;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Security\Core\User\PasswordAuthenticatedUserInterface;

#[ORM\Table(name: "user")]
#[ORM\Entity(repositoryClass: UserRepository::class)]
#[UniqueEntity(fields: "username", message: "Le username est déjà utilisé")]
#[UniqueEntity(fields: "mail", message: "L'email est déjà utilisé")]
#[UniqueEntity(fields: "token", message: "Erreur, token non unique")]

class User implements UserInterface,PasswordAuthenticatedUserInterface
{

    #[ORM\Column(name: "id", type: "integer")]
    #[ORM\Id]
    #[ORM\GeneratedValue(strategy: "AUTO")]
    private $id;

    #[ORM\Column(name: "username", type: "string", length: 191, unique: true)]
    private $username;

    #[ORM\Column(name: "prenom", type: "string", length: 191, nullable: true)]
    private $prenom;

    #[ORM\Column(name: "nom", type: "string", length: 191, nullable: true)]
    private $nom;

    #[ORM\Column(name: "mail", type: "string", length: 191, unique: true)]
    #[Assert\Email()]
    private $mail;

    #[ORM\Column(name: "password", type: "string", length: 191)]
    private $password;

    #[ORM\Column(name: "token", type: "string", length: 191, unique: true)]

    private $token;

    #[ORM\Column(name: "token_validity", type: "datetime")]
    #[Assert\Type("DateTime")]
    private $tokenValidity;

    #[ORM\Column(name: "salt", type: "string", length: 191, nullable: true)]
    private $salt;

    #[ORM\Column(name: "is_active", type: "boolean")]
    private $isActive;

    #[ORM\Column(name: "roles", type: "array")]
    private $roles = array();

    #[ORM\ManyToMany(targetEntity: Film::class, mappedBy: "usersWantToView")]
    private $films;

    #[ORM\ManyToMany(targetEntity: Film::class, mappedBy: "usersWhoSeen")]
    private $filmsVus;

    #[ORM\OneToOne(targetEntity: Profile::class, mappedBy: "user", cascade: ["persist", "remove"])]
    private $profile;

    #[ORM\Column(name: "last_activity", type: "datetime")]
    #[Assert\Type("DateTime")]
    private $lastActivity;

    public function getLastActivity()
    {
        return $this->lastActivity;
    }

    public function setLastActivity($lastActivity)
    {
        $this->lastActivity = $lastActivity;
    }

    public function isActiveNow()
    {
        $delay = new \DateTime('5 minutes ago');

        return ( $this->getLastActivity() > $delay );
    }

    public function getId()
    {
        return $this->id;
    }

    public function setUsername($username)
    {
        $this->username = $username;

        return $this;
    }

    public function getUsername()
    {
        return (string) $this->username;
    }

    public function getUserIdentifier()
    {
        return $this->username;
    }

    public function setActivated($activated)
    {
        $this->isActive = $activated;
    }

    public function getActivated()
    {
        return $this->isActive;
    }

    public function getPrenom()
    {
        return $this->prenom;
    }

    public function setPrenom($prenom)
    {
        $this->prenom = $prenom;
    }

    public function getNom()
    {
        return $this->nom;
    }

    public function setNom($nom)
    {
        $this->nom = $nom;
    }

    public function getMail()
    {
        return $this->mail;
    }

    public function setMail($mail)
    {
        $this->mail = $mail;
    }

    public function setPassword($password)
    {
        $this->password = $password;

        return $this;
    }

    public function getPassword() : ?string
    {
        return $this->password;
    }

    public function setSalt($salt)
    {
        $this->salt = $salt;

        return $this;
    }

    public function getSalt()
    {
        return $this->salt;
    }

    public function getToken()
    {
        return $this->token;
    }

    public function setToken($token)
    {
        //$this->token = hash("sha512", uniqid());
        $this->token = $token;
        $this->setTokenValidity(new \DateTime());

        return $this;
    }

    public function getTokenValidity()
    {
        return $this->tokenValidity;
    }

    private function setTokenValidity(\DateTime $tokenValidity)
    {
        $this->tokenValidity = $tokenValidity;

        return $this;
    }

    public function isValidToken()
    {
        $expire = $this->getTokenValidity()->modify('+1 hour');
        return ( $expire > new \DateTime('now') );
    }

    public function setRoles($roles)
    {
        $this->roles = $roles;

        return $this;
    }

    public function getRoles()
    {
        return $this->roles;
    }

    public function eraseCredentials()
    {

    }

    public function __construct()
    {
        $this->films = new \Doctrine\Common\Collections\ArrayCollection();
        //$this->setToken();
        $this->setLastActivity(new \DateTime('now'));
        $this->setActivated(true);
    }

    ///////////////////////////////////////////////////////////////
   
    public function addFilm(\App\Entity\Film $film)
    {
        $this->films[] = $film;

        return $this;
    }

    public function removeFilm(\App\Entity\Film $film)
    {
        $this->films->removeElement($film);
    }

    public function getFilms()
    {
        return $this->films;
    }

    ///////////////////////////////////////////////////////////////////////////

    public function addFilmVu(\App\Entity\Film $film)
    {
        $this->filmsVus[] = $film;

        return $this;
    }

    public function removeFilmVu(\App\Entity\Film $film)
    {
        $this->filmsVus->removeElement($film);
    }

    public function getFilmsVus()
    {
        return $this->filmsVus;
    }

    public function getNomComplet()
    {
        if ($this->prenom == null && $this->nom == null)
        {
            return $this->getUserIdentifier();
        } else
        return $this->getPrenom()." ".$this->getNom();
    }

    public function isAccountNonExpired()
    {
        return true;
    }

    public function isAccountNonLocked()
    {
        return true;
    }

    public function isCredentialsNonExpired()
    {
        return true;
    }

    public function isEnabled()
    {
        return $this->isActive;
    }

    public function serialize()
    {
        return serialize(array(
            $this->id,
            $this->username,
            $this->password,
            $this->isActive,
            // see section on salt below
            // $this->salt,
        ));
    }

    public function unserialize($serialized)
    {
        list (
            $this->id,
            $this->username,
            $this->password,
            $this->isActive,

            // see section on salt below
            // $this->salt
            ) = unserialize($serialized, array('allowed_classes' => false));
    }

    public function wantToSee(Film $film)
    {
        return $this->getFilms()->contains($film);
    }

    public function haveSeen(Film $film)
    {
        return $this->getFilmsVus()->contains($film);
    }

    public function getProfile(): ?Profile
    {
        return $this->profile;
    }

    public function setProfile(Profile $profile): self
    {
        // set the owning side of the relation if necessary
        if ($profile->getUser() !== $this) {
            $profile->setUser($this);
        }

        $this->profile = $profile;

        return $this;
    }

}
